/**
 *
 * Copyright (c) 2000 Justin E. Forrester (forrestje@bigfoot.com)
 * All Rights Reserved
 * 
 * I provide the Windows "Fuzz" Testing Tool on an AS IS basis, and
 * do not warrant its validity or performance.  I reserve the right
 * to update, modify, or discontinue this software at any time.  I
 * shall have no obligation to supply such updates or modifications or
 * any other form of support to you.
 * 
 * This license is for research uses.  For such uses, there is no
 * charge. I define "research use" to mean you may freely use it
 * inside your organization for whatever purposes you see fit. But you
 * may not re-distribute Fuzz or parts of Fuzz, in any form
 * source or binary (including derivatives), electronic or otherwise,
 * to any other organization or entity without my permission. For
 * other uses, please contact me at forrestje@bigfoot.com. 
 * 
 * All warranties, including without limitation, any warranty of
 * merchantability or fitness for a particular purpose, are hereby
 * excluded.
 * 
 * By your use of Fuzz, you understand and agree that I (or any
 * other person or entity with proprietary rights in Fuzz) am
 * under no obligation to provide either maintenance services,
 * update services, notices of latent defects, or correction of
 * defects for Fuzz.
 * 
 * Even if advised of the possibility of such damages, under no
 * circumstances shall I (or any other person or entity with
 * proprietary rights in the software licensed hereunder) be liable
 * to you or any third party for direct, indirect, or consequential
 * damages of any character regardless of type of action, including,
 * without limitation, loss of profits, loss of use, loss of good
 * will, or computer failure or malfunction.  You agree to indemnify
 * me (and any other person or entity with proprietary rights in the
 * software licensed hereunder) for any and all liability it may
 * incur to third parties resulting from your use of Fuzz.
 *
 **/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <winuser.h>

#include <crtdbg.h>  // for ASSERT macro

//=============================================================================
// MISC CONSTANTS
//=============================================================================
#define LAUNCH_WAIT_TIME                   5000 // sleep time = 5 seconds
#define EVENT_WAIT_TIME                    5   // sleep time in ms
#define CAPTIONLENGTH                      32   // length of caption info field
#define WNDCLASSLENGTH                     32   // length of wnd class

//=============================================================================
// ERROR CODES
//=============================================================================

#ifndef NO_ERROR
#define NO_ERROR                           0
#endif
#define ERR_BAD_LAUNCH                     1
#define ERR_INVALID_CMD_LINE               2
#define ERR_INVALID_ARG                    3
#define ERR_WND_NOT_FOUND                  4
#define ERR_SIZE_FAIL                      5
#define ERR_TOP_LEVEL_NOT_VISIBLE          6
#define ERR_TOP_LEVEL_ZERO_DIM             7
#define ERR_INVALID_PID                    8

// mouse state array indexing
#define LBUTTON                            0
#define RBUTTON                            1

//=============================================================================
// Decls
//=============================================================================

// FUZZ OPTIONS STRUCTURE
//   encapsulates the options given on the command line
typedef struct _FUZZ_OPTIONS {
    BOOL fRandomEvents;           // send fake user events

    BOOL fRandomWin32ViaSendMessage; // use SendMessage API function for tests
    BOOL fRandomWin32ViaPostMessage; // use PostMessage API function for tests
    BOOL fSendWMCommandMessages;    // send only random WM_COMMAND msgs
    BOOL fSendNULLParams;           // use NULL for lparam and wparam in msgs

    DWORD dwNumMessages;            // number of messages to send

    DWORD dwSeed;                   // seed for random number generation

    LPSTR pszAppCmdLine;            // command to use when launching test app
    DWORD dwProcessId;              // process id of process to test
} FUZZ_OPTIONS, *PFUZZ_OPTIONS; 
 
    
//=============================================================================
// Prototypes
//=============================================================================
LONG parseArgs(LONG nNumArgs, LPSTR ppszArgs[], PFUZZ_OPTIONS options);

// launch a process
LONG launchApp(LPSTR pszCommandLine, PROCESS_INFORMATION &pi);

// message sending tests
LONG sendRandomWin32(HWND hwnd, HANDLE hProc, PFUZZ_OPTIONS pOptions);
LONG sendRandomEvents(HWND hwnd, HANDLE hProc, PFUZZ_OPTIONS pOptions);

// fabricated user message helper funcs
LONG generateRandomKeyboardEvent(BOOL *pfKeyStates);
LONG generateRandomMouseEvent(BOOL *pfMouseButtonStates, RECT *pWndDim,
                              DWORD dwScreenHeight, DWORD dwScreenWidth);

// cleaup after the fabricated user message tests
VOID cleanupMouseKeyboardState(VOID);

// determine whether a given key will do unwanted things in the system
BOOL isSystemHotKey(BYTE bVKCode, DWORD dwFlags, BOOL *pfKeyStates);

// command line usage information
VOID printHelp(void);

